
import './utils/browserPolyfills'; // Must be first import
import React, { Suspense } from 'react';
import ReactDOM from 'react-dom/client';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { BrowserRouter } from 'react-router-dom';
import App from './App';
import './index.css';
import { addResourceHints, addInlineCriticalCSS, criticalCSS } from './utils/resourcePriority';
import { initNetworkMonitoring, getNetworkStatus, isLowDataMode } from './utils/networkMonitor';
import { preloadCriticalImages } from './utils/imageOptimizer';
import { AuthProvider } from './contexts/AuthContext';
import { LoadingStrategyProvider } from './components/LoadingStrategyProvider';
import { FontLoadingMonitor } from './components/FontLoadingMonitor';
import { initResourceHints } from './utils/resourceHintsHelper';

// Record timing for performance monitoring
const APP_START_TIME = performance.now();

// Initialize resource hints as early as possible
initResourceHints();

// Add inline critical CSS for faster painting
addInlineCriticalCSS(criticalCSS);

// Initialize network monitoring ONCE with minimal overhead
// Do this early but with lower priority than critical rendering
const networkMonitorInitialized = initNetworkMonitoring(false);

// Early connection quality check to adapt loading strategy
const checkConnectionQuality = () => {
  const networkStatus = getNetworkStatus();
  const isSlow = networkStatus.effectiveType === 'slow-2g' || 
          networkStatus.effectiveType === '2g' || 
          (networkStatus.rtt && networkStatus.rtt > 700);
  
  if (isSlow) {
    document.documentElement.classList.add('slow-connection');
    
    // On very slow connections, we'll avoid a lot of background activities
    if (networkStatus.effectiveType === 'slow-2g') {
      document.documentElement.classList.add('minimal-mode');
    }
  }
  
  return isSlow;
};

const isSlow = checkConnectionQuality();
const isLowData = isLowDataMode();

// Create a single shared query client instance with connection-aware settings
// Enhanced with better timeout handling
const queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      retry: isSlow ? 1 : 2,
      retryDelay: attempt => Math.min(1000 * 2 ** attempt, 30000),
      refetchOnWindowFocus: !isSlow,
      refetchOnMount: !isSlow,
      staleTime: isSlow ? 15 * 60 * 1000 : 5 * 60 * 1000, // 15 min for slow, 5 min for fast
      gcTime: isSlow ? 30 * 60 * 1000 : 15 * 60 * 1000,  // 30 min for slow, 15 min for fast
      networkMode: 'online',
      // Add timeout for very slow connections
      ...(isSlow && {
        queryFn: (context) => {
          const fetchWithTimeout = async (url, timeoutMs = 15000) => {
            const controller = new AbortController();
            const id = setTimeout(() => controller.abort(), timeoutMs);
            try {
              const response = await fetch(url, { signal: controller.signal });
              clearTimeout(id);
              if (!response.ok) throw new Error(`Network error: ${response.status}`);
              return response.json();
            } catch (e) {
              clearTimeout(id);
              throw e;
            }
          };
          return fetchWithTimeout(context.queryKey[0], 15000);
        }
      })
    },
  },
});

// Connection-aware resource hints - only critical ones
addResourceHints({
  preconnect: [
    'https://fonts.googleapis.com',
    'https://fonts.gstatic.com',
    'https://images.unsplash.com',
    'https://storage.googleapis.com'
  ],
  preload: [
    { 
      href: '/lovable-uploads/98a9fcc4-c199-4be7-8bad-6756a3aa8870.png', 
      as: 'image' as 'image'
    },
    // Only preload fonts on faster connections
    ...(isSlow ? [] : [{
      href: 'https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap',
      as: 'style' as 'style'
    }])
  ]
});

// Preload critical logo image - always do this even on slow connections
preloadCriticalImages([
  '/lovable-uploads/98a9fcc4-c199-4be7-8bad-6756a3aa8870.png'
]);

// Basic fallback component for slow connections
const FallbackLoadingComponent = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 p-4">
    <div className="w-20 h-20 mb-4 relative">
      <img 
        src="/lovable-uploads/98a9fcc4-c199-4be7-8bad-6756a3aa8870.png"
        alt="Loading..."
        className="w-full h-full object-contain"
        width="80"
        height="80"
      />
    </div>
    <h2 className="text-xl font-medium mb-2 text-center">LamuLamu</h2>
    <p className="text-gray-500 text-center">Loading, please wait...</p>
    <div className="mt-4 w-full max-w-xs bg-gray-200 h-1.5 rounded-full overflow-hidden">
      <div className="h-full bg-primary animate-pulse rounded-full" style={{width: '60%'}}></div>
    </div>
  </div>
);

// Safer initialization process
function SafeApp() {
  const [hasError, setHasError] = React.useState(false);
  const [errorMessage, setErrorMessage] = React.useState('');
  
  React.useEffect(() => {
    const handleError = (event: ErrorEvent) => {
      console.error('Global error caught:', event.error);
      setHasError(true);
      setErrorMessage(event.message || 'Unknown error occurred');
    };
    
    window.addEventListener('error', handleError);
    return () => window.removeEventListener('error', handleError);
  }, []);
  
  if (hasError) {
    return (
      <div className="min-h-screen flex items-center justify-center bg-gray-100 p-4">
        <div className="bg-white p-6 rounded-lg shadow-md max-w-md w-full">
          <h2 className="text-xl font-bold text-red-600 mb-4">Something went wrong</h2>
          <p className="text-gray-700 mb-4">
            The application encountered an error. Please try refreshing the page.
          </p>
          {errorMessage && (
            <pre className="bg-gray-100 p-3 rounded text-sm overflow-auto max-h-40 mb-4">
              {errorMessage}
            </pre>
          )}
          <button
            onClick={() => window.location.reload()}
            className="w-full bg-blue-600 text-white py-2 px-4 rounded hover:bg-blue-700 transition-colors"
          >
            Refresh Page
          </button>
        </div>
      </div>
    );
  }
  
  return (
    <React.StrictMode>
      <QueryClientProvider client={queryClient}>
        <BrowserRouter>
          <LoadingStrategyProvider>
            <Suspense fallback={<FallbackLoadingComponent />}>
              <FontLoadingMonitor />
              <AuthProvider>
                <App />
              </AuthProvider>
            </Suspense>
          </LoadingStrategyProvider>
        </BrowserRouter>
      </QueryClientProvider>
    </React.StrictMode>
  );
}

// Improved React root creation with better error handling
ReactDOM.createRoot(document.getElementById('root') as HTMLElement).render(
  <SafeApp />
);

// Defer service worker registration based on connection speed
// Remove from App.tsx and only register once here
if (process.env.NODE_ENV === 'production') {
  // Import on demand to avoid import during initial load
  setTimeout(() => {
    import('./utils/serviceWorkerRegistration').then(({ registerServiceWorker }) => {
      window.addEventListener('load', () => {
        // Pass false to allow deferring on slow connections
        registerServiceWorker(false);
      });
    });
  }, isSlow ? 10000 : 2000); // Longer delay for slow connections

}

// Declare requestIdleCallback for TypeScript
declare global {
  interface Window {
    requestIdleCallback: (callback: IdleRequestCallback, options?: IdleRequestOptions) => number;
    __moduleErrorListenerAdded?: boolean;
  }
}
